#
# ddsidl.mk
#
# Rules for generating C++ and Java message classes from IDL using the
# OpenSplice DDS IDL pre-processor/compiler.
#
# Implementation note:  since the classes that are generated do not
# necessarily have the same names as the IDL files that define the messages,
# it's a little hard to map the input IDL files to the output class files when
# building make rules.  So, instead we create a dummy file with an extension
# of ".idlppDone" when we finish processing a given IDL file and use that file
# as the make dependency.
#
# The variables that are used in this file, but need to be defined outside
# of this file, are listed here.
#
# Variables that should be defined in arch_spec_osdds.mk (or its children)
# (of course, these variables can be overridden elsewhere):
# - DDSIDL_COMPILE - the path to the IDL compiler
# - DDSIDL_COMPILE_OPTIONS - the options that should be passed to the compiler
#                            (these should not include the choice of language
#                            for the generated classes - that is selected by
#                            the DDSIDL_GEN_LANG parameter below)
#
# Variables that should be defined in the makefile that includes this file:
# - DDSIDL_IDL_FILES - the list of IDL files to be compiled (including paths)
# - DDSIDL_IDL_DIR - the directory that contains the IDL files (this may
#                    seem redundant since the FILES have path info, but it
#                    is still needed)
# - DDSIDL_INC_DIR - the directory where the generated header files (C++)
#                    should be placed (this variable is unneeded and ignored
#                    when the language choice for the generated code is java)
# - DDSIDL_SRC_DIR - the directory where the generated files should be
#                    placed (for java, this should not include package-based
#                    subdirectories, but for C++ it may, if desired)
# - DDSIDL_GEN_LANG - the language choice for the classes that are generated
#                    ("c++" or "cpp" for C++, or "java" for Java)
# - DDSIDL_EXTRA_INCLUDES - an optional list of packages that should be
#                    "included" in the build process for the current package.
#                    This will pick up IDL files from the extra packages
#                    (that are included in IDL files in the current package)
#                    and pick up IDL-related header files from the extra pkgs.
#                    NOTE that "rms" is assumed to be a needed extra package,
#                    so it doesn't need to be explicitly specified.
#

# build the list of dummy files that we use to track whether each of
# the IDL files have been processed
DDSIDL_GENDONE_SUBDIR := idlGenDone
ddsidl_gendone_path := $(DDSIDL_GENDONE_SUBDIR)
ddsidl_gendone_files := $(subst $(DDSIDL_IDL_DIR),$(ddsidl_gendone_path),\
	$(subst .idl,.idlppDone,$(DDSIDL_IDL_FILES)))

# code generation rules
$(ddsidl_gendone_files) : $(ddsidl_gendone_path)/%.idlppDone : $(DDSIDL_IDL_DIR)/%.idl
	@echo "<**generating DDS datatype classes from IDL**> $(<F)"
	if [ ! -d $(ddsidl_gendone_path) ]; then mkdir $(ddsidl_gendone_path); fi
	@$(DDSIDL_COMPILE) $(DDSIDL_COMPILE_OPTIONS) -l $(DDSIDL_GEN_LANG) \
	    $(foreach extraPkg, $(DDSIDL_EXTRA_INCLUDES) rms, \
	      -I $(SRT_PRIVATE_CONTEXT)/$(extraPkg)/config \
	      -I $(SRT_PUBLIC_CONTEXT)/$(extraPkg)/config) \
	    -I $(DDSIDL_IDL_DIR) -d $(DDSIDL_SRC_DIR) $<
	touch $@
ifneq "$(findstring c, $(DDSIDL_GEN_LANG))" ""
	-rm ddsidl.work 2>/dev/null
	ddsidl_namespace=`grep namespace $(DDSIDL_SRC_DIR)/$(*F).h | awk '{print $$2}'` ; grep DataWriter $(DDSIDL_SRC_DIR)/$(*F)Dcps.h | grep class | grep ';' | sort -u | sed "s/class /declare_dds_message($$ddsidl_namespace::/" | sed "s/;/);/" | sed "s/ //g" | sed "s/DataWriter//" > ddsidl.work
	mv $(DDSIDL_SRC_DIR)/ccpp_$(*F).h $(DDSIDL_SRC_DIR)/ccpp_$(*F).h.true_orig
	(cat $(DDSIDL_SRC_DIR)/ccpp_$(*F).h.true_orig | grep -v \#endif; echo "#include <rms/provider/CETDDS.h>"; cat ddsidl.work; cat $(DDSIDL_SRC_DIR)/ccpp_$(*F).h.true_orig | grep \#endif) > $(DDSIDL_SRC_DIR)/ccpp_$(*F).h
	-rm ddsidl.work 2>/dev/null

	mv $(DDSIDL_SRC_DIR)/$(*F)Dcps_impl.cpp $(DDSIDL_SRC_DIR)/$(*F)Dcps_impl.cpp.true_orig
	echo "#include \"ccpp_$(*F).h\"" > $(DDSIDL_SRC_DIR)/$(*F)Dcps_impl.cpp
	cat $(DDSIDL_SRC_DIR)/$(*F)Dcps_impl.cpp.true_orig >> $(DDSIDL_SRC_DIR)/$(*F)Dcps_impl.cpp
  ifdef DDSIDL_INC_DIR
    ifneq "$(DDSIDL_INC_DIR)" "."
      ifneq "$(DDSIDL_INC_DIR)" "./"
        ifneq "$(DDSIDL_INC_DIR)" "$(DDSIDL_SRC_DIR)"
	@echo "<**relocating DDS headers**> $(subst .idl,.h,$(<F))"
	if [ ! -d $(DDSIDL_INC_DIR) ]; then mkdir $(DDSIDL_INC_DIR); fi
	for myfile in ccpp_$(*F).h $(*F).h $(*F)Dcps.h $(*F)Dcps_impl.h $(*F)SplDcps.h; do \
	  mv $(DDSIDL_SRC_DIR)/$$myfile $(DDSIDL_SRC_DIR)/$$myfile.orig; \
	  cat $(DDSIDL_SRC_DIR)/$$myfile.orig | sed 's,\(#include "\)\(.*$(basename $(notdir $<))\),\1$(PACKAGE)/\2,' > $(DDSIDL_SRC_DIR)/$$myfile; \
	  mv $(DDSIDL_SRC_DIR)/$$myfile $(DDSIDL_INC_DIR); \
	done
	for myfile in $(*F).cpp $(*F)Dcps.cpp $(*F)Dcps_impl.cpp $(*F)SplDcps.cpp; do \
	  mv $(DDSIDL_SRC_DIR)/$$myfile $(DDSIDL_SRC_DIR)/$$myfile.orig; \
	  cat $(DDSIDL_SRC_DIR)/$$myfile.orig | sed 's,\(#include "\)\(.*$(basename $(notdir $<))\),\1$(PACKAGE)/\2,' > $(DDSIDL_SRC_DIR)/$$myfile; \
	done
        endif
      endif
    endif
  endif
endif

.PHONY : codegen_ddsidl
codegen_ddsidl : $(ddsidl_gendone_files)

ifneq "$(findstring java, $(DDSIDL_GEN_LANG))" ""
  # build a list of generated class files that can be cleaned up by starting
  # with a list of all files and removing the ones that have been checked
  # into source control.  If we can't find any CVS "Entries" files, then we
  # don't clean up any files since we can't tell which ones were generated.
  full_gen_file_list := $(shell find $(DDSIDL_SRC_DIR) -type f -name "*.java" -print)
  cvs_entry_file_count := $(shell find $(DDSIDL_SRC_DIR) -type f -name "Entries" -print | wc -l)
  ifneq "$(strip $(full_gen_file_list))" ""
      ifneq ($(cvs_entry_file_count), 0)
          gen_files_to_be_removed := $(foreach file,$(full_gen_file_list),\
              $(shell if [ `grep $(notdir $(file)) -R --include=Entries $(DDSIDL_SRC_DIR)/* | wc -l` == 0 ]; then echo $(file); fi))
      endif
  endif
endif

ifneq "$(findstring c, $(DDSIDL_GEN_LANG))" ""
  # build a list of generated source and header files that can be cleaned up
  gen_files_to_be_removed := \
    $(foreach file, $(basename $(notdir $(DDSIDL_IDL_FILES))), \
      $(DDSIDL_INC_DIR)/ccpp_$(file).h \
      $(DDSIDL_INC_DIR)/$(file).h \
      $(DDSIDL_SRC_DIR)/$(file).cpp \
      $(DDSIDL_INC_DIR)/$(file)Dcps.h \
      $(DDSIDL_SRC_DIR)/$(file)Dcps.cpp \
      $(DDSIDL_INC_DIR)/$(file)Dcps_impl.h \
      $(DDSIDL_SRC_DIR)/$(file)Dcps_impl.cpp \
      $(DDSIDL_INC_DIR)/$(file)SplDcps.h \
      $(DDSIDL_SRC_DIR)/$(file)SplDcps.cpp)
endif

# special cleanup rule
# --> this includes what is in the regular cleanup rule *and* the "clean up
# whatever is not in source control" extras.  This extra piece is a little
# dangerous since new files that haven't yet been checked into source control
# could get deleted by accident.  So, we only do this if specifically requested.
.PHONY : really_clean_ddsidl ddsidl_really_clean
really_clean_ddsidl:
	-rm -f $(ddsidl_gendone_files)
	-rm -f $(DDSIDL_SRC_DIR)/*orig
ifneq "$(strip $(gen_files_to_be_removed))" ""
	-rm -f $(gen_files_to_be_removed)
endif
ddsidl_really_clean : really_clean_ddsidl

# regular cleanup rule
.PHONY : clean_ddsidl
clean_ddsidl:
	-rm -f $(ddsidl_gendone_files)
	-rm -f $(DDSIDL_SRC_DIR)/*orig

# add the local rules to the normal build cycle
codegen: codegen_ddsidl
clean: clean_ddsidl

# include the environment variables for Ddsidl
include SoftRelTools/arch_spec_osdds.mk
