#ifndef MESSAGEBUFFER__H
#define MESSAGEBUFFER__H

#include "DAQMessages/ccpp_NSNMessages.h"
#include <boost/interprocess/containers/map.hpp>
#include <boost/interprocess/containers/vector.hpp>
#include <boost/interprocess/allocators/allocator.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/interprocess/sync/named_mutex.hpp>

namespace novaddt{
  class MessageBuffer;
}


class novaddt::MessageBuffer{
    /// a class to manage point buffer in shared memory
    /// When 10 points are accumulated, we can form an NSNPackedMessage
    public:
        using TimeType = uint64_t;
        using DataType = uint64_t;
        using Point = std::pair<TimeType, DataType>; /// a point is pair (timestamp,nclusters)
        using PointAllocator = boost::interprocess::allocator
                              <Point,boost::interprocess::managed_shared_memory::segment_manager>;
        using PointVector =  boost::interprocess::vector<Point, PointAllocator>;
        using interprocess_mutex = boost::interprocess::interprocess_mutex;
        using scoped_lock = boost::interprocess::scoped_lock<interprocess_mutex>;
        MessageBuffer(const char* shmemName,const char* bufName, size_t Capacity=10):
            _shmemName(shmemName),
            _segment(boost::interprocess::open_or_create, shmemName, 65536),
            _allocator(_segment.get_segment_manager()),
            _buffer(_segment.find_or_construct<PointVector>(bufName)(_allocator)),
            _mutex(_segment.find_or_construct<interprocess_mutex>("mtx")()),
            _capacity(Capacity)
        {
            scoped_lock lock(*_mutex);
            //set buffer length
            _buffer->reserve(_capacity);
            boost::interprocess::managed_shared_memory::shrink_to_fit(shmemName);
            std::cout<<"Open messageBuffer \""<<bufName<<"\" of size= "<<_segment.get_size()<<std::endl;
        }
        
        ~MessageBuffer(){
            scoped_lock lock(*_mutex);
            std::cout<<"clean up!"<<std::endl;
            if(boost::interprocess::shared_memory_object::remove(_shmemName.data())){
                std::cout<<"Shared memory removed"<<std::endl;
            };
        }

        void addPoint(Point p){
            scoped_lock lock(*_mutex);
            if(isFull())
                throw std::logic_error("Trying to fill full buffer");
            _buffer->push_back(p);
        }
        bool isFull(){return (_buffer->size()>=_capacity);}
        size_t size(){return _buffer->size();}
        size_t capacity(){return _capacity;}
        void clear(){_buffer->clear();}
        
        nsnmessages::NSNPackedMessage makeMessage(){
            //prepare a message to be sent
            nsnmessages::NSNPackedMessage m;
            for(size_t i=0;i<_capacity;++i){
                Point p=_buffer->at(i);
                m.time[i]=p.first;
                m.hits[i]=p.second;
            }
            return m;
        }

        nsnmessages::NSNPackedMessage makeMessageAndClear(){
            scoped_lock lock(*_mutex);
            auto m=makeMessage();
            clear();
            return m;
        }
    private: 
        std::string _shmemName;
        boost::interprocess::managed_shared_memory _segment;
        const PointAllocator _allocator;
        PointVector* _buffer;
        interprocess_mutex *_mutex;
        unsigned* _counter;
        size_t _capacity;
};


#endif
